<?php

/**
 * @file
 * The salsaSearchRequest class
 * Used to build the request object.
 */

class SalsaSearchRequest {

  private $document;

  /**
   * Constructor for a salsaSearchRequest class.
   */
  public function __construct() {
    $this->document = new DOMDocument('1.0', 'utf-8');
    $search_request = $this->document->createElement('searchRequest');
    $this->document->appendChild($search_request);
  }

  /**
   * Sets threshold attribute.
   *
   * @param float $threshold
   *   The minimum score of documents returned.
   */
  public function setThreshold($threshold = 0.2) {
    $this->document->documentElement->setAttribute('threshold', $threshold);
  }

  /**
   * Sets the type of the query.
   *
   * @param string $type
   *   Possible values are SENSE, KEYWORD or RELATED
   */
  public function setType($type = 'SENSE') {
    $this->document->documentElement->setAttribute('type', $type);
  }

  /**
   * Defines the total number of elements per page.
   *
   * @param integer $page_size
   *   The number total of elements.
   */
  public function setPageSize($page_size = 10) {
    $this->document->documentElement->setAttribute('pageSize', $page_size);
  }

  /**
   * Sets the current page number.
   *
   * @param integer $page_number
   *   The page number.
   */
  public function setPageNumber($page_number = 0) {
    $this->document->documentElement->setAttribute('pageNumber', $page_number);
  }

  /**
   * Defines the search query.
   *
   * @param string $search
   *   The search query string.
   */
  public function setQuery($search = '') {
    $query = $this->document->createElement('query', $search);
    $this->document->documentElement->appendChild($query);
  }

  /**
   * Defines the constraints for a faceted search.
   *
   * @param array $constraints_list
   *   An associative array containing the constraints.
   */
  public function setConstraints($constraints_list = array()) {
    $constrained = count($constraints_list) ? TRUE : FALSE;
    $this->document->documentElement->setAttribute('constrained', $constrained);
    if ($constrained) {
      $constaints = $this->document->createElement('constraints');
      foreach ($constraints_list as $key => $value) {
        $facet = $this->document->createElement('facet');
        $facet->setAttribute('key', $key);
        $facet->setAttribute('value', $value);
        $constaints->appendChild($facet);
      }
      $this->document->documentElement->appendChild($constaints);
    }
  }

  /**
   * Defines the search facets.
   *
   * @param array $facets_list
   *   An array of facet keys.
   */
  public function setFacets($facets_list = array()) {
    $faceted = count($facets_list) ? TRUE : FALSE;
    $this->document->documentElement->setAttribute('faceted', $faceted);
    if ($faceted) {
      $facets = $this->document->createElement('facets');
      foreach ($facets_list as $value) {
        $key = $this->document->createElement('key', $value);
        $facets->appendChild($key);
      }
      $this->document->documentElement->appendChild($facets);
    }
  }

  /**
   * Helper function to get xml code
   *
   * @return string
   *   The indexable xml string
   */
  public function asXML() {
    return $this->document->saveXML();
  }

}
