<?php

/**
 * @file
 * SearchApiSalsaDevService class
 */

define('SALSADEV_URL', 'api.salsadev.com');

require 'salsaIndexable.inc';
require 'salsaSearchRequest.inc';

class SearchApiSalsaDevService extends SearchApiAbstractService {

  /**
   * Determines whether this service class implementation supports a given
   * feature.
   *
   * @param string $feature
   *   The name of the optional feature.
   *
   * @return boolean
   *   TRUE if this service knows and supports the specified feature. FALSE
   *   otherwise.
   */
  public function supportsFeature($feature) {
    switch (drupal_strtolower($feature)) {
      case 'search_api_facets':
        return TRUE;
    }
    return FALSE;
  }

  /**
   * Form callback called on initialisation for configuration.
   *
   * @return array
   *   A form array for setting service-specific options.
   */
  public function configurationForm(array $form, array &$form_state) {
    if (empty($this->options)) {
      $options = array(
        'username' => 'demo',
        'password' => 'demo2011',
        'searchbox' => 'demo',
        'metakeywords' => FALSE,
      );
    }
    else {
      $options = $this->options;
    }
    $form['username'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#description' => t('Enter you SalsaDev username'),
      '#default_value' => $options['username'],
      '#required' => TRUE,
    );
    $form['password'] = array(
      '#type' => 'textfield',
      '#title' => t('Password'),
      '#description' => t('Enter your SalsaDev password'),
      '#default_value' => $options['password'],
      '#required' => TRUE,
    );
    $form['searchbox'] = array(
      '#type' => 'textfield',
      '#title' => t('Searchbox'),
      '#description' => t('Enter the name of your searchbox.'),
      '#default_value' => $options['searchbox'],
      '#required' => TRUE,
    );
    if (module_exists('metatag')) {
      $form['metakeywords'] = array(
        '#type' => 'checkbox',
        '#title' => t('Override meta keywords ?'),
        '#description' => t('Activate the semantic keywords extraction.'),
        '#default_value' => $options['metakeywords'],
      );
    }
    return $form;
  }


  /**
   * Index the specified items.
   *
   * @param SearchApiIndex $index
   *   The search index for which items should be indexed.
   * @param array $items
   *   An array of items to be indexed, keyed by their id.
   *
   * @return array
   *   An array of the ids of all items that were successfully indexed.
   */
  public function indexItems(SearchApiIndex $index, array $items) {
    $indexed = array();
    foreach ($items as $id => $item) {

      // Dirty hack to update item when reindexing.
      // TODO: replace with the /clean API function.
      $this->deleteItem($id);
      $indexed[] = $this->indexItem($index, $id, $item);

      if ($this->options['metakeywords']) {
        $rawtext = '';
        foreach ($item as $name => $field) {
          if (search_api_is_text_type($field['type'])) {
            $rawtext .= strip_tags($field['value']);
          }
        }
        $node = node_load($id);
        if (is_object($node)) {
          $payload = array(
            'content' => $rawtext,
            'count' => 20,
          );
          $payload = http_build_query($payload);
          $response = $this->salsaRequest('lift/keywords', $payload, 'POST', $type = 'text/plain');
          if ($response->code == 200) {
            $response = new SimpleXMLElement($response->data);
            $tags = array();
            foreach ($response->tag as $tag) {
              $tags[] = (string) $tag['term'];
            }
            $node->metatags['keywords']['value'] = implode(',', $tags);
            node_save($node);
          }
        }
      }

    }
    return $indexed;
  }


  /**
   * Index a single element.
   *
   * @param SearchApiIndex $index
   *   The search index for which item should be indexed.
   * @param integer $id
   *   The document unique identifier.
   * @param array $item
   *   An associative array of fields.
   *
   * @return integer
   *   Returns the unique identifier of indexed element.
   */
  public function indexItem(SearchApiIndex $index, $id, array $item) {
    $indexable = new SalsaIndexable($id);
    foreach ($item as $name => $field) {
      $is_meta = FALSE;
      $meta_data_names = array(
        'title',
        'author',
      );
      $meta_data_types = array(
        'string',
        'integer',
        'decimal',
        'date',
        'duration',
        'boolean',
        'uri',
      );
      if (in_array($name, $meta_data_names) || in_array($field['type'], $meta_data_types)) {
        $is_meta = TRUE;
      }
      if ($is_meta) {
        $value = $field['value'];
        if ($name == 'author') {
          if (is_numeric($field['value'])) {
            $author = user_load($field['value']);
            if ($author) {
              $value = $author->name;
            }
          }
          else {
            $value = 'anonymous';
          }
        }
        $indexable->addMetadata($name, $value);
      }
      else {
        $indexable->addContent($name, $field['value']);
      }
    }
    $response = $this->salsaRequest('document', $indexable->asXML());
    if ($response->code == '200') {
      return $id;
    }
  }

  /**
   * Remove an item from the index.
   *
   * @param integer $id
   *   The document unique identifier.
   */
  public function deleteItem($id) {
    $this->salsaRequest('document/' . $id, NULL, 'DELETE');
  }


  /**
   * Delete items from an index on this server.
   *
   * @param array|string $ids
   *   Either an array containing the ids of the items that should be deleted,
   *   or 'all' if all items should be deleted.
   * @param SearchApiIndex $index
   *   The index from which items should be deleted.
   */
  public function deleteItems($ids = 'all', SearchApiIndex $index = NULL) {
    if ($ids == 'all') {
      // TODO: remove all documents from repository.
    }
    elseif (is_array($ids)) {
      foreach ($ids as $id) {
        $this->salsaRequest('document/' . $id, NULL, 'DELETE');
      }
    }
  }


  /**
   * Executes a search on the server.
   *
   * @param SearchApiQueryInterface $query
   *   The SearchApiQueryInterface object to execute.
   *
   * @return array
   *   An associative array containing the search results, as required by
   *   SearchApiQueryInterface::execute().
   */
  public function search(SearchApiQueryInterface $query) {
    $options = $query->getOptions();

    $offset = isset($options['offset']) ? $options['offset'] : 0;
    $limit = isset($options['limit']) ? $options['limit'] : 1000000;
    $page_number = intval($offset / $limit);

    $search_request = new SalsaSearchRequest();
    $search_request->setPageNumber($page_number);
    $search_request->setPageSize($limit);
    $search_request->setThreshold();
    $search_request->setType('SENSE');

    if ($search_api_facets = $query->getOption('search_api_facets')) {
      $facets = array();
      foreach ($query->getOption('search_api_facets') as $key => $facet) {
        $facets[] = $key;
      }
      $search_request->setFacets($facets);

      $filters = $query->getFilter()->getFilters();
      $constraints = array();
      foreach ($filters as $filter) {
        $constraints[$filter[0]] = $filter[1];
      }
      $search_request->setConstraints($constraints);
    }

    $keys = $query->getKeys();
    unset($keys['#conjunction']);
    if (is_array($keys)) {
      $keys = implode(' ', $keys);
    }
    $search_request->setQuery($keys);

    $results = $this->salsaRequest('search', $search_request->asXML());
    if ($results->code == '200') {
      $results = $this->extractResults($query, $results->data);
    }

    return $results;
  }


  /**
   * Handler function to extracts the results and format output
   * as required by SearchApiQueryInterface::execute().
   *
   * @param SearchApiQueryInterface $query
   *   The SearchApiQueryInterface object.
   * @param object $data
   *   The XML result from SalsaDev.
   *
   * @return array
   *   An associative array containing the search results, as required by
   *   SearchApiQueryInterface::execute().
   */
  protected function extractResults(SearchApiQueryInterface $query, $data) {
    $results = array();
    $doc = new DOMDocument('1.0', 'utf-8');
    $doc->loadXML($data);
    $results['result count'] = $doc->documentElement->getAttribute('total');
    $results['results'] = array();
    if ($results['result count']) {
      $index = $query->getIndex();
      $fields = $index->options['fields'];
      $hits = $doc->getElementsByTagName('hit');
      foreach ($hits as $hit) {
        $id = $hit->getAttribute('uid');
        $result = array(
          'id' => $id,
          'score' => $hit->getAttribute('score'),
          'fields' => array('title' => $hit->getAttribute('title')),
        );
        $result['excerpt'] = $hit->nodeValue;
        $results['results'][$id] = $result;
      }

      $saved_facetgroups = array();
      $facetgroups = $doc->getElementsByTagName('facetgroup');
      foreach ($facetgroups as $facetgroup) {
        $key = $facetgroup->getAttribute('key');
        $facets = $facetgroup->getElementsByTagName('facet');
        $saved_facets = array();
        foreach ($facets as $facet) {
          $saved_facets[] = array(
            'count' => $facet->getAttribute('count'),
            'filter' => '"' . $facet->getAttribute('value') . '"',
          );
        }
        $saved_facetgroups[$key] = $saved_facets;
      }
      $results['search_api_facets'] = $saved_facetgroups;

    }
    return $results;
  }

  /**
   * Executes a SalsaDev request.
   *
   * @param string $function
   *   The API URI function.
   * @param string $payload
   *   A well formatted payload: XML string for application/xml type or
   *   a simple query string build with http_build_query().
   * @param string $method
   *   The method used for the request: GET, POST or DELETE.
   * @param string $type
   *   The request type.
   */
  protected function salsaRequest($function, $payload = NULL, $method = 'POST', $type = 'application/xml') {
    $format = 'http://%1$s:%2$s@%3$s/%4$s?searchbox=%5$s';
    $url = sprintf(
      $format,
      $this->options['username'],
      $this->options['password'],
      SALSADEV_URL,
      $function,
      $this->options['searchbox']
    );
    $options = array(
      'headers' => array('content-type' => $type),
      'method' => $method,
      'data' => $payload,
    );
    return drupal_http_request($url, $options);
  }

}
